# http plugins
%bcond_without geoip
%bcond_without realip
# dns plugins
%bcond_without azure
%bcond_without cloudflare
%bcond_without digitalocean
%bcond_without googlecloud
%bcond_without pdns
%bcond_without rackspace
%bcond_without route53

%if %{with azure}%{with cloudflare}%{with digitalocean}%{with googlecloud}%{with pdns}%{with rackspace}%{with route53}
%bcond_without dnsproviders
%endif

# https://bugzilla.redhat.com/show_bug.cgi?id=995136#c12
%global _dwz_low_mem_die_limit 0

%if %{undefined gobuild}
%define gobuild(o:) go build -buildmode pie -compiler gc -tags="rpm_crashtraceback ${BUILDTAGS:-}" -ldflags "${LDFLAGS:-} -B 0x$(head -c20 /dev/urandom|od -An -tx1|tr -d ' \\n') -extldflags '%__global_ldflags'" -a -v -x %{?**};
%endif

Name:           caddy
Version:        1.0.3
Release:        1%{?dist}
Summary:        Web server with automatic HTTPS
License:        ASL 2.0 and MIT
URL:            https://caddyserver.com
ExclusiveArch:  %{?go_arches:%{go_arches}}%{!?go_arches:%{ix86} x86_64 %{arm} aarch64 ppc64le s390x %{mips}}
%{?go_compiler:BuildRequires: compiler(go-compiler)}
# https://github.com/mholt/caddy/commit/80dfb8b2a7f89b120a627bc4d866a1dc5ed3d92f#diff-04c6e90faac2675aa89e2176d2eec7d8
BuildRequires:  golang >= 1.12
BuildRequires:  systemd
%{?systemd_requires}
Provides:       webserver

# caddy
%global import_path github.com/caddyserver/caddy
# Upstream stopped vendoring dependencies in 1.0.0.  F31+ has those
# dependencies packaged, but in order to update earlier branches we must
# generate our own tarball with vendored dependencies.
#
# git clone https://github.com/caddyserver/caddy.git
# cd caddy
# git checkout v%%{version}
# go mod vendor
# git add vendor
# git commit -m vendor
# git archive --format=tar.gz --prefix=caddy-%%{version}/ HEAD > caddy-%%{version}-vendor.tar.gz
Source0:        caddy-%{version}-vendor.tar.gz
Source1:        caddy.conf
Source2:        caddy.service
Source3:        index.html
# https://github.com/caddyserver/caddy/issues/2584
Patch0:         enable-appVersion-ldflag.patch

# dnsproviders
%global import_path_dnsproviders github.com/caddyserver/dnsproviders
%global version_dnsproviders 0.3.0
Source10:       https://%{import_path_dnsproviders}/archive/v%{version_dnsproviders}/dnsproviders-%{version_dnsproviders}.tar.gz

# lego
%global import_path_lego github.com/go-acme/lego
%global version_lego 2.5.0
Source11:       https://%{import_path_lego}/archive/v%{version_lego}/lego-%{version_lego}.tar.gz

%if %{with azure}
Provides:       bundled(golang(%{import_path_dnsproviders}/azure)) = %{version_dnsproviders}
Provides:       bundled(golang(%{import_path_lego}/providers/dns/azure)) = %{version_lego}
%endif
%if %{with cloudflare}
Provides:       bundled(golang(%{import_path_dnsproviders}/cloudflare)) = %{version_dnsproviders}
Provides:       bundled(golang(%{import_path_lego}/providers/dns/cloudflare)) = %{version_lego}
%endif
%if %{with digitalocean}
Provides:       bundled(golang(%{import_path_dnsproviders}/digitalocean)) = %{version_dnsproviders}
Provides:       bundled(golang(%{import_path_lego}/providers/dns/digitalocean)) = %{version_lego}
%endif
%if %{with googlecloud}
Provides:       bundled(golang(%{import_path_dnsproviders}/googlecloud)) = %{version_dnsproviders}
Provides:       bundled(golang(%{import_path_lego}/providers/dns/gcloud)) = %{version_lego}
%endif
%if %{with pdns}
Provides:       bundled(golang(%{import_path_dnsproviders}/pdns)) = %{version_dnsproviders}
Provides:       bundled(golang(%{import_path_lego}/providers/dns/pdns)) = %{version_lego}
%endif
%if %{with rackspace}
Provides:       bundled(golang(%{import_path_dnsproviders}/rackspace)) = %{version_dnsproviders}
Provides:       bundled(golang(%{import_path_lego}/providers/dns/rackspace)) = %{version_lego}
%endif
%if %{with route53}
Provides:       bundled(golang(%{import_path_dnsproviders}/route53)) = %{version_dnsproviders}
Provides:       bundled(golang(%{import_path_lego}/providers/dns/route53)) = %{version_lego}
%endif

# geoip
%global import_path_geoip github.com/aablinov/caddy-geoip
%global commit_geoip c06787a76821ab7852dc1a5219dd5c05d904dce4
Source20:       https://%{import_path_geoip}/archive/%{commit_geoip}/geoip-%{commit_geoip}.tar.gz
%global import_path_geohash github.com/mmcloughlin/geohash
%global version_geohash 0.9.0
Source21:       https://%{import_path_geohash}/archive/v%{version_geohash}/geohash-%{version_geohash}.tar.gz
%if %{with geoip}
Provides:       bundled(golang(%{import_path_geoip}))
Provides:       bundled(golang(%{import_path_geohash}))
BuildRequires:  golang(github.com/oschwald/maxminddb-golang)
%endif

# realip
%global import_path_realip github.com/captncraig/caddy-realip
%global commit_realip 6df827e22ab8cd5ea90e0844ff29cf62b95127f9
Source22:       https://%{import_path_realip}/archive/%{commit_realip}/realip-%{commit_realip}.tar.gz
%if %{with realip}
Provides:       bundled(golang(%{import_path_realip}))
%endif

# vendored libraries (Source0)
Provides:       bundled(golang(github.com/bifurcation/mint))
Provides:       bundled(golang(github.com/cenkalti/backoff)) = 2.1.1
Provides:       bundled(golang(github.com/cheekybits/genny))
Provides:       bundled(golang(github.com/dustin/go-humanize)) = 1.0.0
Provides:       bundled(golang(github.com/flynn/go-shlex))
Provides:       bundled(golang(github.com/go-acme/lego)) = 2.5.0
Provides:       bundled(golang(github.com/google/uuid)) = 1.1.1
Provides:       bundled(golang(github.com/gorilla/websocket)) = 1.4.0
Provides:       bundled(golang(github.com/hashicorp/go-syslog)) = 1.0.0
Provides:       bundled(golang(github.com/hashicorp/golang-lru))
Provides:       bundled(golang(github.com/jimstudt/http-authentication))
Provides:       bundled(golang(github.com/klauspost/cpuid)) = 1.2.0
Provides:       bundled(golang(github.com/lucas-clemente/aes12))
Provides:       bundled(golang(github.com/lucas-clemente/quic-go)) = 0.10.2
Provides:       bundled(golang(github.com/lucas-clemente/quic-go-certificates))
Provides:       bundled(golang(github.com/mholt/certmagic)) = 0.6.2
Provides:       bundled(golang(github.com/miekg/dns)) = 1.1.3
Provides:       bundled(golang(github.com/naoina/go-stringutil)) = 0.1.0
Provides:       bundled(golang(github.com/naoina/toml)) = 0.1.1
Provides:       bundled(golang(github.com/russross/blackfriday))
Provides:       bundled(golang(golang.org/x/crypto))
Provides:       bundled(golang(golang.org/x/net))
Provides:       bundled(golang(golang.org/x/sys))
Provides:       bundled(golang(golang.org/x/text)) = 0.3.0
Provides:       bundled(golang(gopkg.in/mcuadros/go-syslog.v2)) = 2.2.1
Provides:       bundled(golang(gopkg.in/natefinch/lumberjack.v2)) = 2.0.0
Provides:       bundled(golang(gopkg.in/square/go-jose.v2)) = 2.2.2
Provides:       bundled(golang(gopkg.in/yaml.v2)) = 2.2.2

# vendored libraries (Source11)
Provides:       bundled(golang(github.com/akamai/AkamaiOPEN-edgegrid-golang))
Provides:       bundled(golang(github.com/aliyun/alibaba-cloud-sdk-go))
Provides:       bundled(golang(github.com/aws/aws-sdk-go))
Provides:       bundled(golang(github.com/Azure/azure-sdk-for-go))
Provides:       bundled(golang(github.com/Azure/go-autorest))
Provides:       bundled(golang(github.com/BurntSushi/toml))
Provides:       bundled(golang(github.com/cenkalti/backoff))
Provides:       bundled(golang(github.com/cloudflare/cloudflare-go))
Provides:       bundled(golang(github.com/cpu/goacmedns))
Provides:       bundled(golang(github.com/decker502/dnspod-go))
Provides:       bundled(golang(github.com/dnsimple/dnsimple-go))
Provides:       bundled(golang(github.com/exoscale/egoscale))
Provides:       bundled(golang(github.com/gophercloud/gophercloud))
Provides:       bundled(golang(github.com/iij/doapi))
Provides:       bundled(golang(github.com/JamesClonk/vultr))
Provides:       bundled(golang(github.com/linode/linodego))
Provides:       bundled(golang(github.com/miekg/dns))
Provides:       bundled(golang(github.com/namedotcom/go))
Provides:       bundled(golang(github.com/nrdcg/auroradns))
Provides:       bundled(golang(github.com/nrdcg/goinwx))
Provides:       bundled(golang(github.com/OpenDNS/vegadns2client))
Provides:       bundled(golang(github.com/oracle/oci-go-sdk))
Provides:       bundled(golang(github.com/ovh/go-ovh))
Provides:       bundled(golang(github.com/rainycape/memcache))
Provides:       bundled(golang(github.com/sacloud/libsacloud))
Provides:       bundled(golang(github.com/stretchr/testify))
Provides:       bundled(golang(github.com/timewasted/linode))
Provides:       bundled(golang(github.com/transip/gotransip))
Provides:       bundled(golang(github.com/urfave/cli))
Provides:       bundled(golang(golang.org/x/crypto))
Provides:       bundled(golang(golang.org/x/net))
Provides:       bundled(golang(golang.org/x/oauth2))
Provides:       bundled(golang(google.golang.org/api))
Provides:       bundled(golang(gopkg.in/ns1/ns1-go.v2))
Provides:       bundled(golang(gopkg.in/square/go-jose.v2))


%description
Caddy is the web server with automatic HTTPS.

This package was built with the following plugins:

%{?with_geoip:  http.geoip
}%{?with_realip:  http.realip
}%{?with_azure:  tls.dns.azure
}%{?with_cloudflare:  tls.dns.cloudflare
}%{?with_digitalocean:  tls.dns.digitalocean
}%{?with_googlecloud:  tls.dns.googlecloud
}%{?with_pdns:  tls.dns.powerdns
}%{?with_rackspace:  tls.dns.rackspace
}%{?with_route53:  tls.dns.route53
}


%prep
%setup -q -c -a 10 -a 11 -a 20 -a 21 -a 22

pushd caddy-%{version}
%patch0 -p 1
popd

cp caddy-%{version}/LICENSE.txt LICENSE.txt
cp caddy-%{version}/dist/README.txt README.txt
cp caddy-%{version}/dist/CHANGES.txt CHANGES.txt
mkdir -p $(dirname _build/src/%{import_path})
mv caddy-%{version} _build/src/%{import_path}

%if %{with dnsproviders}
cp dnsproviders-%{version_dnsproviders}/LICENSE LICENSE-dnsproviders
mkdir -p $(dirname _build/src/%{import_path_dnsproviders})
mv dnsproviders-%{version_dnsproviders} _build/src/%{import_path_dnsproviders}
cp lego-%{version_lego}/LICENSE LICENSE-lego
mkdir -p $(dirname _build/src/%{import_path_lego})
mv lego-%{version_lego} _build/src/%{import_path_lego}
%endif

%if %{with geoip}
cp caddy-geoip-%{commit_geoip}/LICENSE LICENSE-geoip
mkdir -p $(dirname _build/src/%{import_path_geoip})
mv caddy-geoip-%{commit_geoip} _build/src/%{import_path_geoip}
cp geohash-%{version_geohash}/LICENSE LICENSE-geohash
mkdir -p $(dirname _build/src/%{import_path_geohash})
mv geohash-%{version_geohash} _build/src/%{import_path_geohash}
%endif

%if %{with realip}
cp caddy-realip-%{commit_realip}/LICENSE LICENSE-realip
mkdir -p $(dirname _build/src/%{import_path_realip})
mv caddy-realip-%{commit_realip} _build/src/%{import_path_realip}
%endif

sed                     -e '/other plugins/ a \\t// plugins added during rpmbuild' \
%{?with_geoip:          -e '/other plugins/ a \\t_ "%{import_path_geoip}"'} \
%{?with_realip:         -e '/other plugins/ a \\t_ "%{import_path_realip}"'} \
%{?with_azure:          -e '/other plugins/ a \\t_ "%{import_path_dnsproviders}/azure"'} \
%{?with_cloudflare:     -e '/other plugins/ a \\t_ "%{import_path_dnsproviders}/cloudflare"'} \
%{?with_digitalocean:   -e '/other plugins/ a \\t_ "%{import_path_dnsproviders}/digitalocean"'} \
%{?with_googlecloud:    -e '/other plugins/ a \\t_ "%{import_path_dnsproviders}/googlecloud"'} \
%{?with_pdns:           -e '/other plugins/ a \\t_ "%{import_path_dnsproviders}/pdns"'} \
%{?with_rackspace:      -e '/other plugins/ a \\t_ "%{import_path_dnsproviders}/rackspace"'} \
%{?with_route53:        -e '/other plugins/ a \\t_ "%{import_path_dnsproviders}/route53"'} \
                        -i _build/src/%{import_path}/caddy/caddymain/run.go


%build
export GOPATH="$PWD/_build:%{gopath}"
export LDFLAGS="${LDFLAGS:-} -X %{import_path}/caddy/caddymain.appVersion=v%{version} "
%gobuild -o _bin/caddy %{import_path}/caddy


%install
install -D -m 0755 _bin/caddy %{buildroot}%{_bindir}/caddy
install -D -m 0644 %{S:1} %{buildroot}%{_sysconfdir}/caddy/caddy.conf
install -D -m 0644 %{S:2} %{buildroot}%{_unitdir}/caddy.service
install -D -m 0644 %{S:3} %{buildroot}%{_datadir}/caddy/index.html
install -d -m 0755 %{buildroot}%{_sysconfdir}/caddy/conf.d
install -d -m 0750 %{buildroot}%{_sharedstatedir}/caddy


%pre
getent group caddy &> /dev/null || \
groupadd -r caddy &> /dev/null
getent passwd caddy &> /dev/null || \
useradd -r -g caddy -d %{_sharedstatedir}/caddy -s /sbin/nologin -c 'Caddy web server' caddy &> /dev/null
exit 0


%post
%systemd_post caddy.service

if [ -x /usr/sbin/getsebool ]; then
    # connect to ACME endpoint to request certificates
    setsebool -P httpd_can_network_connect on
fi
if [ -x /usr/sbin/semanage -a -x /usr/sbin/restorecon ]; then
    # file contexts
    semanage fcontext --add --type httpd_exec_t        '%{_bindir}/caddy'               2> /dev/null || :
    semanage fcontext --add --type httpd_sys_content_t '%{_datadir}/caddy(/.*)?'        2> /dev/null || :
    semanage fcontext --add --type httpd_config_t      '%{_sysconfdir}/caddy(/.*)?'     2> /dev/null || :
    semanage fcontext --add --type httpd_var_lib_t     '%{_sharedstatedir}/caddy(/.*)?' 2> /dev/null || :
    restorecon -r %{_bindir}/caddy %{_datadir}/caddy %{_sysconfdir}/caddy %{_sharedstatedir}/caddy || :
fi
if [ -x /usr/sbin/semanage ]; then
    # QUIC
    semanage port --add --type http_port_t --proto udp 80   2> /dev/null || :
    semanage port --add --type http_port_t --proto udp 443  2> /dev/null || :
    # HTTP challenge alternate port
    semanage port --add --type http_port_t --proto tcp 5033 2> /dev/null || :
fi


%preun
%systemd_preun caddy.service


%postun
%systemd_postun_with_restart caddy.service

if [ $1 -eq 0 ]; then
    if [ -x /usr/sbin/getsebool ]; then
        # connect to ACME endpoint to request certificates
        setsebool -P httpd_can_network_connect off
    fi
    if [ -x /usr/sbin/semanage ]; then
        # file contexts
        semanage fcontext --delete --type httpd_exec_t        '%{_bindir}/caddy'               2> /dev/null || :
        semanage fcontext --delete --type httpd_sys_content_t '%{_datadir}/caddy(/.*)?'        2> /dev/null || :
        semanage fcontext --delete --type httpd_config_t      '%{_sysconfdir}/caddy(/.*)?'     2> /dev/null || :
        semanage fcontext --delete --type httpd_var_lib_t     '%{_sharedstatedir}/caddy(/.*)?' 2> /dev/null || :
        # QUIC
        semanage port     --delete --type http_port_t --proto udp 80   2> /dev/null || :
        semanage port     --delete --type http_port_t --proto udp 443  2> /dev/null || :
        # HTTP challenge alternate port
        semanage port     --delete --type http_port_t --proto tcp 5033 2> /dev/null || :
    fi
fi


%files
%license LICENSE.txt
%{?with_geoip:%license LICENSE-geoip}
%{?with_geoip:%license LICENSE-geohash}
%{?with_realip:%license LICENSE-realip}
%{?with_dnsproviders:%license LICENSE-dnsproviders}
%{?with_dnsproviders:%license LICENSE-lego}
%doc README.txt CHANGES.txt
%{_bindir}/caddy
%{_datadir}/caddy
%{_unitdir}/caddy.service
%dir %{_sysconfdir}/caddy
%dir %{_sysconfdir}/caddy/conf.d
%config(noreplace) %{_sysconfdir}/caddy/caddy.conf
%attr(0750,caddy,caddy) %dir %{_sharedstatedir}/caddy


%changelog
* Wed Oct 23 2019 Carl George <carl@george.computer> - 1.0.3-1
- Update to v1
- Remove dyn, gandi, namecheap, and rfc2136 dns providers
- Add patch0 to fix `-version` flag

* Wed Jul 24 2019 Fedora Release Engineering <releng@fedoraproject.org> - 0.11.4-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_31_Mass_Rebuild

* Thu May 09 2019 Carl George <carl@george.computer> - 0.11.4-2
- Switch unit file from ProtectSystem strict to full rhbz#1706651

* Wed Mar 06 2019 Carl George <carl@george.computer> - 0.11.4-1
- Latest upstream
- Update bundled dnsproviders to 0.1.3
- Update bundled lego to 2.2.0
- Enable googlecloud, route53, and azure dns providers on epel7
- Allow custom http port with default config file rhbz#1685446

* Thu Jan 31 2019 Fedora Release Engineering <releng@fedoraproject.org> - 0.11.1-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_30_Mass_Rebuild

* Wed Nov 14 2018 Carl George <carl@george.computer> - 0.11.1-2
- Buildrequires at least golang 1.10

* Tue Nov 13 2018 Carl George <carl@george.computer> - 0.11.1-1
- Latest upstream
- Update bundled geoip

* Fri Oct 19 2018 Carl George <carl@george.computer> - 0.11.0-3
- Enable httpd_can_network_connect selinux boolean to connect to ACME endpoint rhbz#1641158
- Define UDP 80/443 as selinux http_port_t for QUIC rhbz#1608548
- Define TCP 5033 as selinux http_port_t for HTTP challenge rhbz#1641160

* Thu Jul 12 2018 Fedora Release Engineering <releng@fedoraproject.org> - 0.11.0-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_29_Mass_Rebuild

* Sat May 12 2018 Carl George <carl@george.computer> - 0.11.0-1
- Latest upstream

* Sat Apr 21 2018 Carl George <carl@george.computer> - 0.10.14-1
- Latest upstream
- Overhaul %%prep to extract everything with %%setup
- Edit lego providers to require acmev2 instead of acme
- Add provides for specific providers from %%import_path_dnsproviders and %%import_path_lego
- Add azure dns provider on f28+

* Fri Apr 20 2018 Carl George <carl@george.computer> - 0.10.11-6
- Enable geoip plugin on EL7
- Only provide bundled geoip/realip/dnsproviders/lego when the respective plugin is enabled

* Wed Apr 18 2018 Carl George <carl@george.computer> - 0.10.11-5
- Add geoip plugin

* Tue Apr 17 2018 Carl George <carl@george.computer> - 0.10.11-4
- Correct ExclusiveArch fallback

* Mon Apr 16 2018 Carl George <carl@george.computer> - 0.10.11-3
- Enable s390x
- Disable googlecloud and route53 dns providers on EL7 due to dependency issues

* Fri Mar 30 2018 Carl George <carl@george.computer> - 0.10.11-2
- Add googlecloud dns provider
- Add route53 dns provider
- Set minimum golang version to 1.9
- Set selinux labels in scriptlets

* Sat Feb 24 2018 Carl George <carl@george.computer> - 0.10.11-1
- Latest upstream

* Sat Feb 24 2018 Carl George <carl@george.computer> - 0.10.10-4
- Change ProtectSystem from strict to full in unit file on RHEL

* Wed Feb 07 2018 Fedora Release Engineering <releng@fedoraproject.org> - 0.10.10-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_28_Mass_Rebuild

* Thu Jan 11 2018 Carl George <carl@george.computer> - 0.10.10-2
- Add powerdns provider

* Mon Oct 09 2017 Carl George <carl@george.computer> - 0.10.10-1
- Latest upstream

* Mon Oct 02 2017 Carl George <carl@george.computer> - 0.10.9-6
- Add provides for bundled libraries

* Mon Oct 02 2017 Carl George <carl@george.computer> - 0.10.9-5
- Enable rfc2136 dns provider
- List plugins in description

* Mon Sep 18 2017 Carl George <carl@george.computer> - 0.10.9-4
- Exclude s390x

* Sun Sep 17 2017 Carl George <carl@george.computer> - 0.10.9-3
- Add realip plugin
- Add conditionals for plugins

* Sat Sep 16 2017 Carl George <carl@george.computer> - 0.10.9-2
- Add sources for caddyserver/dnsproviders and xenolf/lego
- Disable all dns providers that require additional libraries (dnsimple, dnspod, googlecloud, linode, ovh, route53, vultr)
- Rewrite default index.html

* Tue Sep 12 2017 Carl George <carl@george.computer> - 0.10.9-1
- Latest upstream
- Add config validation to unit file
- Disable exoscale dns provider https://github.com/xenolf/lego/issues/429

* Fri Sep 08 2017 Carl George <carl@george.computer> - 0.10.8-1
- Latest upstream
- Build with %%gobuild macro
- Move config subdirectory from /etc/caddy/caddy.conf.d to /etc/caddy/conf.d

* Tue Aug 29 2017 Carl George <carl@george.computer> - 0.10.7-1
- Latest upstream

* Fri Aug 25 2017 Carl George <carl@george.computer> - 0.10.6-2
- Use SIQQUIT to stop service
- Increase the process limit from 64 to 512
- Only `go get` in caddy/caddymain

* Fri Aug 11 2017 Carl George <carl@george.computer> - 0.10.6-1
- Latest upstream
- Add webserver virtual provides
- Drop tmpfiles and just own /var/lib/caddy directly
- Remove PrivateDevices setting from unit file, it prevents selinux process transitions
- Disable rfc2136 dns provider https://github.com/caddyserver/dnsproviders/issues/11

* Sat Jun 03 2017 Carl George <carl.george@rackspace.com> - 0.10.3-2
- Rename Envfile to envfile
- Rename Caddyfile to caddy.conf
- Include additional configs from caddy.conf.d directory

* Fri May 19 2017 Carl George <carl.george@rackspace.com> - 0.10.3-1
- Latest upstream

* Mon May 15 2017 Carl George <carl.george@rackspace.com> - 0.10.2-1
- Initial package
